import { Declare, SingleData, Widget } from 'ptnl-constructor-sdk';

import {
    digitCapacityDefault,
    digitCapacityMax,
    i18N,
    numIndicatorsToShow,
} from './constants';
import { EBlockKey, EViewOption } from './enum';
import * as echarts from 'echarts/core';
import { LineChart, LineSeriesOption, ScatterChart } from 'echarts/charts';
import { Text } from 'ptnl-constructor-sdk/config';
import {
    TooltipComponent,
    TooltipComponentOption,
    GridComponent,
    GridComponentOption,
    LegendComponent,
    LegendComponentOption,
    MarkPointComponent,
    MarkLineComponent,
    DataZoomComponent,
    ToolboxComponent,
    TitleComponent,
    
} from 'echarts/components';
import { SunburstChart } from 'echarts/charts';
import { SVGRenderer } from 'echarts/renderers';
import { registerLayout, setCanvasCreator } from 'echarts/core';
import { yDigitsGroup } from './view-settings';
import { ColumnType, SortDirection } from 'ptnl-constructor-sdk/data';

echarts.use([
    TitleComponent,
    LineChart,
    ScatterChart,
    TooltipComponent,
    GridComponent,
    LegendComponent,
    MarkPointComponent,
    MarkLineComponent,
    DataZoomComponent,
    ToolboxComponent,
    SVGRenderer,
    SunburstChart
]);
type ECOption = echarts.ComposeOption<
    | LineSeriesOption
    | TooltipComponentOption
    | GridComponentOption
    | LegendComponentOption
>;
type ECharts = ReturnType<typeof echarts.init>;
const i18n: Record<string, Text> = {
    'no-value': {
        ru: 'Не найдено',
        en: 'No data',
    },
};
@Declare({
    provideCssVariables: true,
})
export class IndicatorGrad extends Widget implements SingleData {
    readonly data!: SingleData['data'];
    readonly dataSettings!: SingleData['dataSettings'];

    private check_is_all_null(row:any,keys:any){
        for(let k of keys){
            if(!(row[k]==undefined)){
                return false;
            }
        }
        return true;
    }
    private recursiveSort(arr:any[],lvl=0){
        
        let cur_col=this.dataSettings.columnsByBlock[EBlockKey.Series][lvl];
        if(cur_col==undefined){
            return;
        }
        //console.log(this.dataSettings.sort,cur_col)
        let sort_settings=this.dataSettings.sort.filter(x=>x.column.id==cur_col.id);
        if(sort_settings.length>0){
          //  console.log("Сортируем",cur_col,sort_settings[0])
            arr.sort((a,b)=>{
                if(cur_col.type==ColumnType.Number){
                    if(sort_settings[0].direction==SortDirection.Asc){
                        return parseFloat(a.name)-parseFloat(b.name);
                    }else{
                        return parseFloat(a.name)-parseFloat(b.name);
                    }
                }
                if(cur_col.type==ColumnType.String){
                    if(sort_settings[0].direction==SortDirection.Asc){
                        return (a.name>b.name)?1:-1;
                    }else{
                        return (a.name>b.name)?-1:1;
                    }
                }   
                if(cur_col.type==ColumnType.Date){
                    if(sort_settings[0].direction==SortDirection.Asc){
                        return (new Date(a.name)>new Date(b.name))?1:-1;
                    }else{
                        return (new Date(a.name)>new Date(b.name))?-1:1;
                    }
                }
                if(sort_settings[0].direction==SortDirection.Asc){
                    return (a.name>b.name)?1:-1;
                }else{
                    return (a.name>b.name)?-1:1;
                }   
            })
        }
        for(let c of arr){
            this.recursiveSort(c.children,lvl+1)
        }
    }
    private buildData(){
        let ret:any[]=[];
        let value_col=this.dataSettings.columnsByBlock[EBlockKey.Value][0].path;
        for(let row of this.data){
            let c_lvl:{children:any[],value?:number}={
                children:ret,
            }
            for(let c_num=0;c_num<this.dataSettings.columnsByBlock[EBlockKey.Series].length;c_num++ ){
                let col=this.dataSettings.columnsByBlock[EBlockKey.Series][c_num];
                if(row[col.path]==undefined){
                    if(c_num==this.dataSettings.columnsByBlock[EBlockKey.Series].length-1){
                        break;
                    }
                    if(this.check_is_all_null(row,this.dataSettings.columnsByBlock[EBlockKey.Series].slice(c_num+1))){
                        break;
                    }
                }
                let obj_array=c_lvl.children.filter(x=>x.name==row[col.path]);
                if(obj_array.length==0){
                    c_lvl.children.push({
                        name:row[col.path],
                        children:[]
                    })
                    obj_array=[c_lvl.children[c_lvl.children.length-1]];
                }
                c_lvl=obj_array[0]
            }
            c_lvl["value"]=row[value_col];
        }
        this.recursiveSort(ret);
        return ret;
    }
    private format_num(num:number){
        let opt={};
        if(this.viewSettings[EViewOption.EnableDigitCapacity]){
            opt={
                maximumFractionDigits:parseInt(this.viewSettings[EViewOption.DigitCapacity]),
                minimumFractionDigits:parseInt(this.viewSettings[EViewOption.DigitCapacity])
            }
        }
        for(let item of yDigitsGroup){
            const [value, ru, en,mul] = item;
            if(this.viewSettings[EViewOption.NumberGroups]==value)
                return (new Intl.NumberFormat('ru-Ru',opt)).format(num/mul)+((value=='')?(value):(' '+value))
        }
    }
    private myChart:any;
    private renderWidget(): void {
        var chartDom = document.getElementById('root');
        this.myChart = echarts.init(chartDom!);
        let data = this.buildData();
        console.log(data);
          let option = {
            color:this.theme.colors,
            tooltip:{
                formatter:(params:any)=>{
                    let val=" ";
                    if(this.viewSettings[EViewOption.ShowFactName]){
                        val+=params.name;
                    }
                    if(this.viewSettings[EViewOption.ShowValue]){
                        val+="<br>"+(new Intl.NumberFormat('ru-Ru',{})).format(params.value);
                    }
                    return val;
                }

            },
            series: {
              type: 'sunburst',
              data: data,
              nodeClick:false,
              tooltip:{
                show:this.viewSettings[EViewOption.Tooltip]
              },
              
              sort:(a:any,b:any)=>{
                let col=this.dataSettings.columnsByBlock[EBlockKey.Series][a.depth-1];
                if(this.dataSettings.sort.filter(x=>x.column.id==col.id)){
                    return a.dataIndex-b.dataIndex;
                }else{
                    return a.getValue()-b.getValue();
                }

            },
              radius: [this.viewSettings[EViewOption.Radius], '90%'],
              label: {
                rotate: 'radial',
                show:true,
                formatter:(params:any)=>{
                    let val=" ";
                    if(this.viewSettings[EViewOption.ShowFactName]){
                        val+=params.name;
                    }
                    if(this.viewSettings[EViewOption.ShowValue]){
                        val+="\n"+this.format_num(params.value);
                    }
                    return val;
                }

              }
            }
          };
          this.myChart.setOption(option);
          
        window.removeEventListener('resize', this.chartResize);
        window.addEventListener('resize', this.chartResize);

    }

    onLangChange(): void {
        this.onChange();
    }

    onThemeChange(): void {
        this.renderWidget();
    }

    onChange(): void {
        this.renderWidget();
        this.ready();
    }

    private chartResize = () => {
        if (this.myChart) {
            
            this.myChart.resize();
        }
    };
}
