import { DataSettings, ViewSettings } from 'ptnl-constructor-sdk';
import {
    checkbox,
    colorPicker,
    CreateViewSettings,
    input,
    select,
    textArea,
    ViewSettingsItem,
    ViewSettingsValidation,
} from 'ptnl-constructor-sdk/config';
import { digitCapacityMax } from './constants';
import { EViewOption } from './enum';

export const yDigitsGroup:[string,string,string,number][] = [
    ['', 'Не отображать', 'Not set',1],
    ['тыс.', 'тыс.', 'thousand',1000],
    ['млн', 'млн', 'million',1000000],
    ['млрд', 'млрд', 'billion',1000000000]
];


export const createViewSettings: CreateViewSettings<DataSettings> = ({
    dataSettings,
    viewSettings,
}: {
    dataSettings: DataSettings;
    viewSettings: ViewSettings;
}) => {
    return [
        checkbox({
            key: EViewOption.ShowFactName,
            label: {
                ru: 'Включить отображение названия сектора',
                en: 'Включить отображение названия сектора',
            },
            defaultValue: true,
        }),
        checkbox({
            key: EViewOption.ShowValue,
            label: {
                ru: 'Включить отображение абсолютного значения',
                en: 'Включить отображение абсолютного значения',
            },
            defaultValue: true,
        }),
        checkbox({
            key: EViewOption.Tooltip,
            label: {
                ru: 'Всплывающие подсказки',
                en: 'Tooltip',
            },
            defaultValue: true,
        }),
        select({
            key: EViewOption.NumberGroups,
            label: {
                ru: 'Отображать суффикс для групп разрядов',
                en: 'Axis values',
            },
            defaultValue: yDigitsGroup[0][0],
            options: yDigitsGroup.map((item) => {
                const [value, ru, en,mul] = item;
                return {
                    label: {
                        ru,
                        en,
                    },
                    value,
                };
            }),
        }),    
        checkbox({
            key: EViewOption.EnableDigitCapacity,
            label: {
                ru: 'Включить разрядность значений',
                en: 'Enable digit capacity',
            },
            defaultValue: true,
        }),
        input({
            key: EViewOption.DigitCapacity,
            label: {
                ru: 'Разрядность значений',
                en: 'Digit capacity',
            },
            defaultValue: `2`,
        }),
        input({
            key: EViewOption.Radius,
            label: {
                ru: 'Радиус внутренней окружности',
                en: 'Radius',
            },
            defaultValue: '0',
        }),

       
    ];
};


export const validation: ViewSettingsValidation = {
    [EViewOption.DigitCapacity]: (value) => {
        const valueFloat = parseFloat(value);
        if (
            !/^\d+$/.test(value) ||
            valueFloat < 0 ||
            valueFloat > digitCapacityMax
        ) {
            return {
                ru: `укажите целое число от 0 до ${digitCapacityMax}`,
                en: `enter an integer from 0 to ${digitCapacityMax}`,
            };
        }
        return null;
    },
    [EViewOption.IndicatorStepCount]:(value)=>{
        const valueFloat = parseFloat(value);
        if (
            !/^\d+$/.test(value) ||
            valueFloat < 2 
        ) {
            return {
                ru: `укажите целое число больше 2`,
                en: `enter an integer from 0 `,
            };
        }
        return null;
    }
};
